==> wcag-algorithms-1.20.1-sources.jar: org/verapdf/wcag/algorithms/entities/DFSTreeNodeIterator.java <==
/**
 * This file is part of veraPDF wcag algorithms, a module of the veraPDF project.
 * Copyright (c) 2015, veraPDF Consortium <info@verapdf.org>
 * All rights reserved.
 *
 * veraPDF wcag algorithms is free software: you can redistribute it and/or modify
 * it under the terms of either:
 *
 * The GNU General public license GPLv3+.
 * You should have received a copy of the GNU General Public License
 * along with veraPDF wcag algorithms as the LICENSE.GPL file in the root of the source
 * tree.  If not, see http://www.gnu.org/licenses/ or
 * https://www.gnu.org/licenses/gpl-3.0.en.html.
 *
 * The Mozilla Public License MPLv2+.
 * You should have received a copy of the Mozilla Public License along with
 * veraPDF wcag algorithms as the LICENSE.MPL file in the root of the source tree.
 * If a copy of the MPL was not distributed with this file, you can obtain one at
 * http://mozilla.org/MPL/2.0/.
 */
package org.verapdf.wcag.algorithms.entities;

import java.util.Iterator;

class DFSTreeNodeIterator implements Iterator<INode> {

	private final INode root;
	private int childIndex;
	private DFSTreeNodeIterator innerCurrentIterator;

	public DFSTreeNodeIterator(INode root) {
		this.root = root;
		this.childIndex = -1;
		nextInnerIterator();
	}

	@Override
	public boolean hasNext() {
		return innerCurrentIterator != null;
	}

	@Override
	public INode next() {
		if (innerCurrentIterator == null) {
			return null;
		}
		if (!innerCurrentIterator.hasNext()) {
			nextInnerIterator();
			if (!innerCurrentIterator.hasNext()) {
				innerCurrentIterator = null;
				return root;
			}
		}

		return innerCurrentIterator.next();
	}

	private void nextInnerIterator() {
		if (root == null) {
			return;
		}
		++childIndex;
		if (childIndex < root.getChildren().size()) {
			innerCurrentIterator = new DFSTreeNodeIterator(root.getChildren().get(childIndex));
		} else {
			innerCurrentIterator = new DFSTreeNodeIterator(null);
		}
	}
}

==> wcag-algorithms-1.28.2-sources.jar: org/verapdf/wcag/algorithms/entities/DFSTreeNodeIterator.java <==
/*
 * This file is part of veraPDF wcag algorithms, a module of the veraPDF project.
 * Copyright (c) 2015, veraPDF Consortium <info@verapdf.org>
 * All rights reserved.
 *
 * veraPDF wcag algorithms is free software: you can redistribute it and/or modify
 * it under the terms of either:
 *
 * The GNU General public license GPLv3+.
 * You should have received a copy of the GNU General Public License
 * along with veraPDF wcag algorithms as the LICENSE.GPL file in the root of the source
 * tree.  If not, see http://www.gnu.org/licenses/ or
 * https://www.gnu.org/licenses/gpl-3.0.en.html.
 *
 * The Mozilla Public License MPLv2+.
 * You should have received a copy of the Mozilla Public License along with
 * veraPDF wcag algorithms as the LICENSE.MPL file in the root of the source tree.
 * If a copy of the MPL was not distributed with this file, you can obtain one at
 * http://mozilla.org/MPL/2.0/.
 */
package org.verapdf.wcag.algorithms.entities;

import java.util.Iterator;

public class DFSTreeNodeIterator implements Iterator<INode> {

	private final INode root;
	private int childIndex;
	private DFSTreeNodeIterator innerCurrentIterator;

	public DFSTreeNodeIterator(INode root) {
		this.root = root;
		this.childIndex = -1;
		nextInnerIterator();
	}

	@Override
	public boolean hasNext() {
		return innerCurrentIterator != null;
	}

	@Override
	public INode next() {
		if (innerCurrentIterator == null) {
			return null;
		}
		if (!innerCurrentIterator.hasNext()) {
			nextInnerIterator();
			if (!innerCurrentIterator.hasNext()) {
				innerCurrentIterator = null;
				return root;
			}
		}

		return innerCurrentIterator.next();
	}

	private void nextInnerIterator() {
		if (root == null) {
			return;
		}
		++childIndex;
		if (childIndex < root.getChildren().size()) {
			innerCurrentIterator = new DFSTreeNodeIterator(root.getChildren().get(childIndex));
		} else {
			innerCurrentIterator = new DFSTreeNodeIterator(null);
		}
	}
}
